#include "drawingsetdialog.h"
#include "ui_drawingsetdialog.h"

DrawingSetDialog::DrawingSetDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DrawingSetDialog)
{
    ui->setupUi(this);
    setWindowModality(Qt::ApplicationModal);
    setWindowFlags(Qt::CustomizeWindowHint | Qt::WindowCloseButtonHint);

    QIntValidator *validator1 = new QIntValidator(1, 10, this);
    ui->lineEdit_lineWidth->setValidator(validator1);

    QIntValidator *validator2 = new QIntValidator(50, 3030, this);
    ui->lineEdit_paperWidth->setValidator(validator2);

    QIntValidator *validator3 = new QIntValidator(0, 1300, this);
    ui->lineEdit_right->setValidator(validator3);

    QIntValidator *validator4 = new QIntValidator(0, 500, this);
    ui->lineEdit_mark->setValidator(validator4);

    QIntValidator *validator5 = new QIntValidator(20, 1000, this);
    ui->lineEdit_butt->setValidator(validator5);
}

DrawingSetDialog::~DrawingSetDialog()
{
    delete ui;
}

void DrawingSetDialog::refreshLanguage()
{
    ui->retranslateUi(this);
}

void DrawingSetDialog::refreshConfigPara(QString filePath)
{
    m_configfile = filePath;
    QSettings settings(filePath, QSettings::IniFormat);

    bool rotate90 = settings.value("DrawSet/rotate90").toBool();//是否正向旋转90度
    bool vecfont = settings.value("DrawSet/vecfont").toBool();//是否使用矢量字体
    QString lineWidth = settings.value("DrawSet/linewidth").toString();//线宽
    QString paperWidth = settings.value("DrawSet/paperwidth").toString();//纸张宽度
    QString rightMargin = settings.value("DrawSet/rightmargin").toString();//右边距
    QString markMargin = settings.value("DrawSet/markmargin").toString();//马克间距
    QString buttMargin = settings.value("DrawSet/buttmargin").toString();//对接符间距

    ui->checkBox_angle->setChecked(rotate90);
    ui->checkBox_font->setChecked(vecfont);
    ui->lineEdit_lineWidth->setText(lineWidth);
    ui->lineEdit_paperWidth->setText(paperWidth);
    ui->lineEdit_right->setText(rightMargin);
    ui->lineEdit_mark->setText(markMargin);
    ui->lineEdit_butt->setText(buttMargin);
}

double DrawingSetDialog::getRotateAngle()
{
    double angle = 0;
    bool bl = ui->checkBox_angle->isChecked();
    if(bl == true)
    {
        angle = 270;
    }
    return angle;
}

void DrawingSetDialog::on_pushButton_cancel_clicked()
{
    done(0);
}

void DrawingSetDialog::on_pushButton_ok_clicked()
{
    QSettings settings(m_configfile, QSettings::IniFormat);

    int rotate90 = ui->checkBox_angle->isChecked();//是否正向旋转90度
    int vecfont = ui->checkBox_font->isChecked();//是否使用矢量字体
    QString lineWidth = ui->lineEdit_lineWidth->text();//线宽
    QString paperWidth = ui->lineEdit_paperWidth->text();//纸张宽度
    QString rightMargin = ui->lineEdit_right->text();//右边距
    QString markMargin = ui->lineEdit_mark->text();//马克间距
    QString buttMargin = ui->lineEdit_butt->text();//对接符间距

    settings.setValue("DrawSet/rotate90",rotate90);
    settings.setValue("DrawSet/vecfont",vecfont);
    settings.setValue("DrawSet/linewidth",lineWidth);
    settings.setValue("DrawSet/paperwidth",paperWidth);
    settings.setValue("DrawSet/rightmargin",rightMargin);
    settings.setValue("DrawSet/markmargin",markMargin);
    settings.setValue("DrawSet/buttmargin",buttMargin);

    done(1);
}
